==> xl-scheduler-2015.6.7-sources.jar: com/xebialabs/xlplatform/scheduler/ControlTaskExecutor.scala <==
package com.xebialabs.xlplatform.scheduler

import java.io.File
import java.util.{HashMap => JHashMap, List => JList, Map => JMap}

import akka.actor.{Actor, ActorLogging, Props}
import com.xebialabs.deployit.checks.Checks._
import com.xebialabs.deployit.engine.tasker._
import com.xebialabs.deployit.plugin.api.flow.Step
import com.xebialabs.deployit.plugin.api.reflect.MethodDescriptor
import com.xebialabs.deployit.plugin.api.udm.{ConfigurationItem, Parameters}
import com.xebialabs.deployit.repository.{RepositoryService, WorkDir, WorkDirFactory}
import com.xebialabs.deployit.security.PermissionEnforcer.ROLE_ADMIN
import com.xebialabs.deployit.security.Permissions
import com.xebialabs.deployit.task.WorkdirCleanerTrigger
import com.xebialabs.xlplatform.scheduler.ControlTaskExecutor.Messages.InvokeControlTask
import com.xebialabs.xlplatform.scheduler.spring.ServiceHolder
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.core.authority.SimpleGrantedAuthority
import org.springframework.security.core.context.SecurityContextHolder

import scala.collection.convert.wrapAll._
import scala.language.postfixOps

object ControlTaskExecutor {
  def props() = Props(classOf[ControlTaskExecutor])

  object Messages {

    case class InvokeControlTask(configurationItemId: String, controlTaskName: String, controlTaskParams: JMap[String, String])

  }

}

class ControlTaskExecutor extends Actor with ActorLogging with ControlTaskService with WorkDirSupport with Impersonation {
  implicit val executionContext = context.dispatcher

  val repositoryService = ServiceHolder.getRepositoryService
  val taskExecutionEngine = ServiceHolder.getTaskExecutionEngine
  val workDirFactory = ServiceHolder.getWorkDirFactory

  override def receive: Receive = {
    case m: InvokeControlTask => invokeTask(m.configurationItemId, m.controlTaskName, m.controlTaskParams)
  }

  private[scheduler] def invokeTask(configurationItemId: String, controlTaskName: String, controlTaskParams: JMap[String, String]): Unit = {
    impersonateAs("system", roles = List(ROLE_ADMIN)) {
      val controlTask = prepareControlTask(configurationItemId, controlTaskName, Option(controlTaskParams).getOrElse(new JHashMap[String, String]()))
      withWorkDir() { implicit workDir =>
        val taskId = createTask(controlTask)
        executeTask(taskId)
      }
    }
  }
}

case class ControlTaskHolder(ci: ConfigurationItem, methodDescriptor: MethodDescriptor, parameters: Parameters)

trait ControlTaskService {
  val TASK_TYPE: String = "CONTROL"

  val repositoryService: RepositoryService
  val taskExecutionEngine: TaskExecutionEngine

  def prepareControlTask(configurationItemId: String, controlTaskName: String, controlTaskParams: JMap[String, String]): ControlTaskHolder = {
    val ci = repositoryService.read[ConfigurationItem](configurationItemId)

    val methodDescriptor = ci.getType.getDescriptor.getControlTask(controlTaskName)
    checkArgument(methodDescriptor != null, s"ConfigurationItem ${ci.getId} of type ${ci.getType} does not have a control task named $controlTaskName.")

    val parameters = Option(methodDescriptor.getParameterObjectType) match {
      case Some(paramsType) =>
        val params = paramsType.getDescriptor.newInstance("parameters").asInstanceOf[Parameters]
        controlTaskParams.foreach(p => params.setProperty(p._1, p._2))
        params
      case _ => null
    }

    ControlTaskHolder(ci, methodDescriptor, parameters)
  }

  def createTask(controlTask: ControlTaskHolder)(implicit workDir: WorkDir): String = {

    val steps = controlTask.methodDescriptor.invoke[JList[Any]](controlTask.ci, controlTask.parameters).map {
      case s: Step => new TaskStep(s)
      case unknown => throw new IllegalArgumentException(s"Control tasks with step type ${unknown.getClass} are not supported, please use flow.Step type.")
    }

    val description = s"Control task [${controlTask.methodDescriptor.getName}}] for ${controlTask.ci.getId}"
    val taskSpec = new TaskSpecification(description, Permissions.getAuthentication, BlockBuilders.steps(description, null, steps).build(), null, new File(workDir.getPath), false, false, true)

    val metadata = taskSpec.getMetadata
    metadata.put("taskType", TASK_TYPE)
    metadata.put("taskName", controlTask.methodDescriptor.getName)
    metadata.put("controlTaskTargetCI", controlTask.ci.getId)

    taskSpec.getListeners.add(new WorkdirCleanerTrigger(workDir))

    taskExecutionEngine.register(taskSpec)
  }

  def executeTask(taskId: String): Unit = {
    taskExecutionEngine.execute(taskId)
  }

  def retrieveTask(taskId: String): Task = {
    taskExecutionEngine.retrieve(taskId)
  }

  def cancelTask(taskId: String): Unit = {
    taskExecutionEngine.cancel(taskId)
  }

  def archiveTask(taskId: String): Unit = {
    taskExecutionEngine.archive(taskId)
  }

}

trait WorkDirSupport {
  val workDirFactory: WorkDirFactory

  def withWorkDir(prefix: String = "task")(func: WorkDir => Unit): Unit = {
    implicit val workDir = workDirFactory.newWorkDir(prefix)
    try {
      func(workDir)
    } catch {
      case e: Exception =>
        workDir.delete()
        throw e
    }
  }
}

trait Impersonation {
  def impersonateAs(username: String, password: String = "", roles: List[String] = List())(func: => Unit): Unit = {
    val context = SecurityContextHolder.getContext
    val origAuthentication = context.getAuthentication
    try {
      context.setAuthentication(new UsernamePasswordAuthenticationToken(username, password, roles.map(new SimpleGrantedAuthority(_))))
      func
    } finally {
      context.setAuthentication(origAuthentication)
    }
  }
}

==> xl-scheduler-2017.4.3-sources.jar: com/xebialabs/xlplatform/scheduler/ControlTaskExecutor.scala <==
package com.xebialabs.xlplatform.scheduler

import java.util.{HashMap => JHashMap, List => JList, Map => JMap}

import akka.actor.{Actor, ActorLogging, Props}
import com.xebialabs.deployit.checks.Checks._
import com.xebialabs.deployit.engine.tasker._
import com.xebialabs.deployit.plugin.api.flow.Step
import com.xebialabs.deployit.plugin.api.reflect.MethodDescriptor
import com.xebialabs.deployit.plugin.api.udm.{ConfigurationItem, Parameters}
import com.xebialabs.deployit.repository.{RepositoryService, WorkDir, WorkDirFactory}
import com.xebialabs.deployit.security.PermissionEnforcer.ROLE_ADMIN
import com.xebialabs.deployit.security.Permissions
import com.xebialabs.deployit.task.WorkdirCleanerTrigger
import com.xebialabs.xlplatform.scheduler.ControlTaskExecutor.Messages.InvokeControlTask
import com.xebialabs.xlplatform.scheduler.spring.ServiceHolder
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.core.authority.SimpleGrantedAuthority
import org.springframework.security.core.context.SecurityContextHolder

import scala.collection.convert.wrapAll._
import scala.language.postfixOps

object ControlTaskExecutor {
  def props() = Props(classOf[ControlTaskExecutor])

  object Messages {

    case class InvokeControlTask(configurationItemId: String, controlTaskName: String, controlTaskParams: JMap[String, String])

  }

}

class ControlTaskExecutor extends Actor with ActorLogging with ControlTaskService with WorkDirSupport with Impersonation {
  implicit val executionContext = context.dispatcher

  val repositoryService = ServiceHolder.getRepositoryService
  val taskExecutionEngine = ServiceHolder.getTaskExecutionEngine
  val workDirFactory = ServiceHolder.getWorkDirFactory

  override def receive: Receive = {
    case m: InvokeControlTask => invokeTask(m.configurationItemId, m.controlTaskName, m.controlTaskParams)
  }

  private[scheduler] def invokeTask(configurationItemId: String, controlTaskName: String, controlTaskParams: JMap[String, String]): Unit = {
    impersonateAs("system", roles = List(ROLE_ADMIN)) {
      val controlTask = prepareControlTask(configurationItemId, controlTaskName, Option(controlTaskParams).getOrElse(new JHashMap[String, String]()))
      withWorkDir() { implicit workDir =>
        val taskId = createTask(controlTask)
        executeTask(taskId)
      }
    }
  }
}

case class ControlTaskHolder(ci: ConfigurationItem, methodDescriptor: MethodDescriptor, parameters: Parameters)

trait ControlTaskService {
  val TASK_TYPE: String = "CONTROL"

  val repositoryService: RepositoryService
  val taskExecutionEngine: TaskExecutionEngine

  def prepareControlTask(configurationItemId: String, controlTaskName: String, controlTaskParams: JMap[String, String]): ControlTaskHolder = {
    val ci = repositoryService.read[ConfigurationItem](configurationItemId)

    val methodDescriptor = ci.getType.getDescriptor.getControlTask(controlTaskName)
    checkArgument(methodDescriptor != null, s"ConfigurationItem ${ci.getId} of type ${ci.getType} does not have a control task named $controlTaskName.")

    val parameters = Option(methodDescriptor.getParameterObjectType) match {
      case Some(paramsType) =>
        val params = paramsType.getDescriptor.newInstance("parameters").asInstanceOf[Parameters]
        controlTaskParams.foreach(p => params.setProperty(p._1, p._2))
        params
      case _ => null
    }

    ControlTaskHolder(ci, methodDescriptor, parameters)
  }

  def createTask(controlTask: ControlTaskHolder)(implicit workDir: WorkDir): String = {

    val steps = controlTask.methodDescriptor.invoke[JList[Any]](controlTask.ci, controlTask.parameters).map {
      case s: Step => new TaskStep(s)
      case unknown => throw new IllegalArgumentException(s"Control tasks with step type ${unknown.getClass} are not supported, please use flow.Step type.")
    }

    val description = s"Control task [${controlTask.methodDescriptor.getName}}] for ${controlTask.ci.getId}"
    val taskSpec = new TaskSpecification(description, Permissions.getAuthentication, workDir, BlockBuilders.steps(description, null, steps).build(), null, false, false, true)

    val metadata = taskSpec.getMetadata
    metadata.put("taskType", TASK_TYPE)
    metadata.put("taskName", controlTask.methodDescriptor.getName)
    metadata.put("taskLabel", controlTask.methodDescriptor.getLabel)
    metadata.put("controlTaskTargetCI", controlTask.ci.getId)

    taskSpec.getListeners.add(new WorkdirCleanerTrigger(workDir))

    taskExecutionEngine.register(taskSpec)
  }

  def executeTask(taskId: String): Unit = {
    taskExecutionEngine.execute(taskId)
  }

  def retrieveTask(taskId: String): Task = {
    taskExecutionEngine.retrieve(taskId)
  }

  def cancelTask(taskId: String): Unit = {
    taskExecutionEngine.cancel(taskId)
  }

  def archiveTask(taskId: String): Unit = {
    taskExecutionEngine.archive(taskId)
  }

}

trait WorkDirSupport {
  val workDirFactory: WorkDirFactory

  def withWorkDir(prefix: String = "task")(func: WorkDir => Unit): Unit = {
    implicit val workDir = workDirFactory.newWorkDir(prefix)
    try {
      func(workDir)
    } catch {
      case e: Exception =>
        workDir.delete()
        throw e
    }
  }
}

trait Impersonation {
  def impersonateAs(username: String, password: String = "", roles: List[String] = List())(func: => Unit): Unit = {
    val context = SecurityContextHolder.getContext
    val origAuthentication = context.getAuthentication
    try {
      context.setAuthentication(new UsernamePasswordAuthenticationToken(username, password, roles.map(new SimpleGrantedAuthority(_))))
      func
    } finally {
      context.setAuthentication(origAuthentication)
    }
  }
}
